"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var awilix_router_core_1 = require("awilix-router-core");
var invokers_1 = require("./invokers");
var koa_compose_1 = __importDefault(require("koa-compose"));
var Router = require('@koa/router');
/**
 * Registers one or multiple decorated controller classes.
 *
 * @param ControllerClass One or multiple "controller" classes
 *        with decorators to register
 */
function controller(ControllerClass) {
    var router = new Router();
    if (Array.isArray(ControllerClass)) {
        ControllerClass.forEach(function (c) {
            return _registerController(router, awilix_router_core_1.getStateAndTarget(c));
        });
    }
    else {
        _registerController(router, awilix_router_core_1.getStateAndTarget(ControllerClass));
    }
    return koa_compose_1.default([router.routes(), router.allowedMethods()]);
}
exports.controller = controller;
/**
 * Loads controllers for the given pattern.
 *
 * @param pattern
 * @param opts
 */
function loadControllers(pattern, opts) {
    var router = new Router();
    awilix_router_core_1.findControllers(pattern, __assign({}, opts, { absolute: true })).forEach(_registerController.bind(null, router));
    return koa_compose_1.default([router.routes(), router.allowedMethods()]);
}
exports.loadControllers = loadControllers;
/**
 * Reads the config state and registers the routes in the router.
 *
 * @param router
 * @param ControllerClass
 */
function _registerController(router, stateAndTarget) {
    if (!stateAndTarget) {
        return;
    }
    var state = stateAndTarget.state, target = stateAndTarget.target;
    /*tslint:disable-next-line*/
    var invoker = invokers_1.makeInvoker(target);
    var rolledUp = awilix_router_core_1.rollUpState(state);
    rolledUp.forEach(function (methodCfg, methodName) {
        methodCfg.verbs.forEach(function (httpVerb) {
            var method = httpVerb.toLowerCase();
            if (httpVerb === awilix_router_core_1.HttpVerbs.ALL) {
                method = 'all';
            }
            router[method].apply(router, [methodCfg.paths].concat(methodCfg.beforeMiddleware, [invoker(methodName)], methodCfg.afterMiddleware));
        });
    });
}
//# sourceMappingURL=controller.js.map